//
//  ImageLoop.m
//  ImageLoop
//
//  Created by lanou on 16/1/22.
//  Copyright © 2016年 lanou. All rights reserved.
//

#import "ImageLoop.h"

@interface ImageLoop () <UIScrollViewDelegate>

{
    UIScrollView *_scrollView;
    UIImageView *_leftImageView;
    UIImageView *_centerImageView;
    UIImageView *_rightImageView;
    NSArray *_imageArray;
    NSTimer *_timer;
    NSTimeInterval _ti;
    NSInteger _leftIndex;
    NSInteger _centerIndex;
    NSInteger _rightIndex;
    NSInteger _width;
    NSInteger _height;
}

@end

@implementation ImageLoop

- (instancetype)initWithFrame:(CGRect)frame imageArray:(NSArray *)array position:(Position)position;
{
    self = [super initWithFrame:frame];
    if (self) {
        _width = frame.size.width;
        _height = frame.size.height;
        _currentPage = 0;
        _leftIndex = array.count - 1;
        _centerIndex = 0;
        _rightIndex = 1;
        _imageArray = array;
        
        _scrollView = [[UIScrollView alloc] initWithFrame:self.bounds];
        _scrollView.contentSize = CGSizeMake(_width * 3, 0);
        _scrollView.pagingEnabled = YES;
        _scrollView.showsHorizontalScrollIndicator = NO;
        _scrollView.delegate = self;
        [_scrollView setContentOffset:CGPointMake(_width, 0) animated:NO];
        
        _leftImageView = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, _width, _height)];
        [_leftImageView setImage:[array objectAtIndex:array.count - 1]];
        
        _centerImageView = [[UIImageView alloc] initWithFrame:CGRectMake(_width, 0, _width, _height)];
        [_centerImageView setImage:[array objectAtIndex:0]];
        
        _rightImageView = [[UIImageView alloc] initWithFrame:CGRectMake(_width * 2, 0, _width, _height)];
        [_rightImageView setImage:[array objectAtIndex:1]];
        
        _pageControll = [[UIPageControl alloc] initWithFrame:CGRectMake(_width * 3 / 4, _height - 30, _width / 4, 30)];
        _pageControll.numberOfPages = _imageArray.count;
        _pageControll.pageIndicatorTintColor = [UIColor whiteColor];
        _pageControll.currentPageIndicatorTintColor = [UIColor redColor];
        _position = position;
        
        _timer = [NSTimer scheduledTimerWithTimeInterval:5.0 target:self selector:@selector(changePage) userInfo:nil repeats:YES];
        _ti = _timer.timeInterval;
        
        [_scrollView addSubview:_leftImageView];
        [_scrollView addSubview:_centerImageView];
        [_scrollView addSubview:_rightImageView];
        [self addSubview:_scrollView];
        [self addSubview:_pageControll];
    }
    return self;
}

- (void)changePage
{
    if (_position == Right) {
        [_scrollView setContentOffset:CGPointMake(_width * 2, 0) animated:YES];
        _leftIndex = _centerIndex;
        _centerIndex = (_centerIndex + 1) % _imageArray.count;
        _rightIndex = (_centerIndex + 1) % _imageArray.count;
    }else{
        [_scrollView setContentOffset:CGPointMake(0, 0) animated:YES];
        _rightIndex = _centerIndex;
        _centerIndex = (_centerIndex - 1 + _imageArray.count) % _imageArray.count;
        _leftIndex = (_centerIndex - 1 + _imageArray.count) % _imageArray.count;
    }
    [_centerImageView setImage:[_imageArray objectAtIndex:_centerIndex]];
    [_scrollView setContentOffset:CGPointMake(_width, 0) animated:NO];
    [_leftImageView setImage:[_imageArray objectAtIndex:_leftIndex]];
    [_rightImageView setImage:[_imageArray objectAtIndex:_rightIndex]];
    _currentPage = _centerIndex;
    _pageControll.currentPage = _centerIndex;
}

- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView
{
    _ti = _timer.timeInterval;
    [_timer invalidate];
    if (scrollView.contentOffset.x != _width) {
        if (scrollView.contentOffset.x == _width * 2) {
            _leftIndex = _centerIndex;
            _centerIndex = (_centerIndex + 1) % _imageArray.count;
            _rightIndex = (_centerIndex + 1) % _imageArray.count;
        }else if (scrollView.contentOffset.x == 0){
            _rightIndex = _centerIndex;
            _centerIndex = (_centerIndex - 1 + _imageArray.count) % _imageArray.count;
            _leftIndex = (_centerIndex - 1 + _imageArray.count) % _imageArray.count;
        }
        [_centerImageView setImage:[_imageArray objectAtIndex:_centerIndex]];
        [_scrollView setContentOffset:CGPointMake(_width, 0) animated:NO];
        [_leftImageView setImage:[_imageArray objectAtIndex:_leftIndex]];
        [_rightImageView setImage:[_imageArray objectAtIndex:_rightIndex]];
        _currentPage = _centerIndex;
        _pageControll.currentPage = _centerIndex;
    }
    _timer = [NSTimer scheduledTimerWithTimeInterval:_ti target:self selector:@selector(changePage) userInfo:nil repeats:YES];
}

- (void)setPosition:(Position)position
{
    _ti = _timer.timeInterval;
    [_timer invalidate];
    _position = position;
    _timer = [NSTimer scheduledTimerWithTimeInterval:_ti target:self selector:@selector(changePage) userInfo:nil repeats:YES];
}

- (void)setCurrentPage:(NSInteger)currentPage
{
    [_timer invalidate];
    _currentPage = currentPage;
    _pageControll.currentPage = _currentPage;
    _centerIndex = _currentPage;
    _leftIndex = (_centerIndex - 1 + _imageArray.count) % _imageArray.count;
    _rightIndex = (_currentPage + 1) % _imageArray.count;
    [_centerImageView setImage:[_imageArray objectAtIndex:_centerIndex]];
    [_scrollView setContentOffset:CGPointMake(_width, 0) animated:NO];
    [_leftImageView setImage:[_imageArray objectAtIndex:_leftIndex]];
    [_rightImageView setImage:[_imageArray objectAtIndex:_rightIndex]];
    _timer = [NSTimer scheduledTimerWithTimeInterval:_ti target:self selector:@selector(changePage) userInfo:nil repeats:YES];
}

@end
